@@ -22,23 +22,30 @@ package nokogiri.internals.c14n;
 /**
  * @author Christian Geuer-Pollmann
  */
-public class Canonicalizer20010315OmitComments extends Canonicalizer20010315 {
+public class Canonicalizer20010315OmitComments extends Canonicalizer20010315
+{
 
-    /**
-     * Constructor Canonicalizer20010315WithXPathOmitComments
-     *
-     */
-    public Canonicalizer20010315OmitComments() {
-        super(false);
-    }
+  /**
+   * Constructor Canonicalizer20010315WithXPathOmitComments
+   *
+   */
+  public
+  Canonicalizer20010315OmitComments()
+  {
+    super(false);
+  }
 
-    /** @inheritDoc */
-    public final String engineGetURI() {
-        return Canonicalizer.ALGO_ID_C14N_OMIT_COMMENTS;
-    }
+  /** @inheritDoc */
+  public final String
+  engineGetURI()
+  {
+    return Canonicalizer.ALGO_ID_C14N_OMIT_COMMENTS;
+  }
 
-    /** @inheritDoc */
-    public final boolean engineGetIncludeComments() {
-        return false;
-    }
+  /** @inheritDoc */
+  public final boolean
+  engineGetIncludeComments()
+  {
+    return false;
+  }
 }
